/*****************************************************************************
* File Name: initializer.c
*
* Version 2.19
*
* Description: This source file contains routines to populate the emulated 
* file system with status files.
*
* Owner:
*	Ranjith M, Applications Engineer (rnjt@cypress.com)
*   Ivan Grytsyk, Applications Engineer Sr (gryt@cypress.com)
*   Vasyl Rodionov, Staff Systems Engr (rodi@cypress.com)
*
* Related Document:
* 	None
*
* Hardware Dependency:
* 	CY8CKIT-042, 
*   CY3280-MBR3, 
*   CY8CKIT-193,
*   CY3679 Atlas EVK,
*   CY8CKIT-044,
*   CY8CKIT-046,
*   CY8CKIT-042 BLE,
*   CY3676  Hercules EVK,
*   CY3677  Hercules EVK
*
* Code Tested With:
* 	Creator 3.3 CP3(3.3.0.9604)
*	GCC 4.9-2015-q1-update(Cypress provided)
*	CY8CKIT-044, -046, -042, -042 BLE, BLE Dongle, -040, -042 BLE-A
*
******************************************************************************
* Copyright (2016), Cypress Semiconductor Corporation.
******************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*****************************************************************************/
#include "project.h"
#include "USBFS_msc_cust.h"
#include "general.h"

extern uint8 emulatedMemory[USBFS_MSC_EM_IMAGE_SIZE];


CYCODE unsigned char readyMsg[] = {'R','E','A','D','Y',0x0D,0x0A};


/*******************************************************************************
* Function Name: SetStatus()
********************************************************************************
* Summary:
*  Writes the status message in the emulated file.
*
* Parameters:
*  const unsigned char* statusMessage: Pointer to the appropriate status message
*  to be displayed in the emulated file.
*
* Return:
*  void
*
*******************************************************************************/
void SetStatus(const unsigned char* statusMessage)
{
	uint16 memIndex = ZERO;
	uint8 msgIndex = ZERO;

	memIndex = emulatedMemory[0xA20 + 0x1C] - STATUS_MSG_SIZE;
	
	/* File contents are updated here after programming operation. */
	/* Status message. */
	for(msgIndex = 0; msgIndex < STATUS_MSG_SIZE; msgIndex++, memIndex++)
	{
		emulatedMemory[0xC00 + memIndex] = statusMessage[msgIndex];
	}
	
	/* Set the flag to force a fault so that 
	 * the OS reads the file system again. */
	forceOS = TRUE;
}

/*******************************************************************************
* Function Name: InitMassStorage()
********************************************************************************
* Summary:
*  Initializes the SRAM memory with an emulated file system. The addresses are 
*  hardcoded and need not be changed.
*
* Parameters:
*  void
*
* Return:
*  void
*
*******************************************************************************/
void InitMassStorage(void)
{
	uint16 memIndex = ZERO;
	uint8 msgSize = ZERO;
	uint8 msgIndex = ZERO;
	
	for(memIndex = 0xC00; memIndex < USBFS_MSC_EM_IMAGE_SIZE; memIndex++)
	{
		/* Fill the emulated memory with spaces. */
		emulatedMemory[memIndex] = 0x20;
	}
	
	emulatedMemory[0x00] = 0xEB;
	emulatedMemory[0x01] = 0x3C;
	emulatedMemory[0x02] = 0x90;
	
	emulatedMemory[0x0C] = 0x02;
	emulatedMemory[0x0D] = 0x01;
	emulatedMemory[0x0E] = 0x01;
	
	emulatedMemory[0x10] = 0x01;
	emulatedMemory[0x11] = 0x10;
	emulatedMemory[0x13] = 0x40;
	emulatedMemory[0x14] = 0x0B;
	emulatedMemory[0x15] = 0xF8;
	emulatedMemory[0x16] = 0x04;
	emulatedMemory[0x18] = 0x12;
	emulatedMemory[0x1A] = 0x01;
	
	emulatedMemory[0x1FE] = 0x55;
	emulatedMemory[0x1FF] = 0xAA;
	
	emulatedMemory[0x200] = 0xF8;
	emulatedMemory[0x201] = 0xFF;
	emulatedMemory[0x202] = 0xFF;
	emulatedMemory[0x203] = 0xFF;
	emulatedMemory[0x204] = 0x0F;
	
	/* Mass Storage Device Name */	
	emulatedMemory[0xA00] = 'H';
	emulatedMemory[0xA01] = 'i';
	emulatedMemory[0xA02] = 'm';
	emulatedMemory[0xA03] = 'a';
	emulatedMemory[0xA04] = ' ';
	emulatedMemory[0xA05] = ' ';
	emulatedMemory[0xA06] = ' ';
	emulatedMemory[0xA07] = ' ';
	emulatedMemory[0xA08] = ' ';
	emulatedMemory[0xA09] = ' ';
	emulatedMemory[0xA0A] = ' ';
	emulatedMemory[0xA0B] = 0x28;
	
	/******************* Status file *****************/
	emulatedMemory[0xA20] = 'S';
	emulatedMemory[0xA21] = 'T';
	emulatedMemory[0xA22] = 'A';
	emulatedMemory[0xA23] = 'T';
	emulatedMemory[0xA24] = 'U';
	emulatedMemory[0xA25] = 'S';
	emulatedMemory[0xA26] = ' ';
	emulatedMemory[0xA27] = ' ';
	emulatedMemory[0xA28] = 'T';
	emulatedMemory[0xA29] = 'X';
	emulatedMemory[0xA2A] = 'T';
	emulatedMemory[0xA2B] = 0x01;

	/* File attributes - Created data etc. */
	emulatedMemory[0xA2D] = 0x0E;
	emulatedMemory[0xA2E] = 0xEB;
	emulatedMemory[0xA2F] = 0x7C;
	
	emulatedMemory[0xA30] = 0xB6;
	emulatedMemory[0xA31] = 0x42;
	emulatedMemory[0xA32] = 0xB6;
	emulatedMemory[0xA33] = 0x42;
	emulatedMemory[0xA36] = 0x6E;
	emulatedMemory[0xA37] = 0x7B;
	
	emulatedMemory[0xA38] = 0xB6;
	emulatedMemory[0xA39] = 0x42;
	
	/* File Location */
	emulatedMemory[0xA3A] = 0x02;
	emulatedMemory[0xA3B] = 0x00;
	
	/* File Contents */
	memIndex = 0x00;
	

	
	/* Write the ready message */
	msgSize = sizeof(readyMsg);
	for(msgIndex = ZERO; msgIndex < msgSize; msgIndex++, memIndex++)
		emulatedMemory[0xC00 + memIndex] = readyMsg[msgIndex];
    
	/* File Size */
	emulatedMemory[0xA3C] = msgSize;
	
	
	forceOS = FALSE;
	statusFileTimer = ZERO;
}


/* [] END OF FILE */
